@@ -482,12 +482,9 @@ gdm_exec_wait (char * const *argv, gboolean no_display,
 
 	pid = gdm_fork_extra ();
 	if (pid == 0) {
-		int i;
-
 		closelog ();
 
-		for (i = 0; i < sysconf (_SC_OPEN_MAX); i++)
-			close (i);
+		gdm_close_all_descriptors (0 /* from */, -1 /* except */);
 
 		/* No error checking here - if it's messed the best response
 		 * is to ignore & try to continue */
@@ -930,4 +927,54 @@ gdm_test_opt (const char *cmd, const char *help, const char *option)
 	fclose (fp);
 	return got_it;
 }
+
+int
+gdm_fdgetc (int fd)
+{
+	char buf[1];
+	int bytes;
+
+	bytes = read (fd, buf, 1);
+	if (bytes != 1)
+		return EOF;
+	else
+		return (int)buf[0];
+}
+
+char *
+gdm_fdgets (int fd)
+{
+	int c;
+	int bytes = 0;
+	GString *gs = g_string_new (NULL);
+	for (;;) {
+		c = gdm_fdgetc (fd);
+		if (c == '\n')
+			return g_string_free (gs, FALSE);
+		/* on EOF */
+		if (c < 0) {
+			if (bytes == 0) {
+				g_string_free (gs, TRUE);
+				return NULL;
+			} else {
+				return g_string_free (gs, FALSE);
+			}
+		} else {
+			bytes ++;
+			g_string_append_c (gs, c);
+		}
+	}
+}
+
+void
+gdm_close_all_descriptors (int from, int except)
+{
+	int i;
+	int max = sysconf (_SC_OPEN_MAX);
+	for (i = from; i < max; i++) {
+		if (i != except)
+			close(i);
+	}
+}
+
 /* EOF */
