@@ -317,15 +317,17 @@ setup_automatic_session (GdmDisplay *display, const char *name)
 	greet = FALSE;
 	gdm_debug ("setup_automatic_session: Automatic login: %s", login);
 
-	if ( ! gdm_verify_setup_user (display, login, display->name))
-		return FALSE;
-
 	/* Run the init script. gdmslave suspends until script
 	 * has terminated */
 	gdm_slave_exec_script (display, GdmDisplayInit, NULL, NULL);
 
 	gdm_debug ("setup_automatic_session: DisplayInit script finished");
 
+	if ( ! gdm_verify_setup_user (display, login, display->name))
+		return FALSE;
+
+	gdm_debug ("setup_automatic_session: Automatic login successful");
+
 	return TRUE;
 }
 
@@ -449,23 +451,27 @@ gdm_slave_run (GdmDisplay *display)
 
 	    check_notifies_now ();
     }
+
+    /* We can use d->handled from now on on this display,
+     * since the lookup was done in server start */
     
     gnome_setenv ("XAUTHORITY", d->authfile, TRUE);
     gnome_setenv ("DISPLAY", d->name, TRUE);
 
-    /* Now the display name and hostname is final */
-    if ( ! ve_string_empty (GdmAutomaticLogin)) {
-	    g_free (ParsedAutomaticLogin);
-	    ParsedAutomaticLogin = gdm_parse_enriched_login (GdmAutomaticLogin,
-							     display);
-    }
+    if (d->handled) {
+	    /* Now the display name and hostname is final */
+	    if ( ! ve_string_empty (GdmAutomaticLogin)) {
+		    g_free (ParsedAutomaticLogin);
+		    ParsedAutomaticLogin = gdm_parse_enriched_login (GdmAutomaticLogin,
+								     display);
+	    }
 
-    if ( ! ve_string_empty (GdmTimedLogin)) {
-	    g_free (ParsedTimedLogin);
-	    ParsedTimedLogin = gdm_parse_enriched_login (GdmTimedLogin,
-							 display);
+	    if ( ! ve_string_empty (GdmTimedLogin)) {
+		    g_free (ParsedTimedLogin);
+		    ParsedTimedLogin = gdm_parse_enriched_login (GdmTimedLogin,
+								 display);
+	    }
     }
-
     
     /* X error handlers to avoid the default one (i.e. exit (1)) */
     do_xfailed_on_xio_error = TRUE;
@@ -484,7 +490,8 @@ gdm_slave_run (GdmDisplay *display)
     else
 	    maxtries = 10;
     
-    while (openretries < maxtries &&
+    while (d->handled &&
+	   openretries < maxtries &&
 	   d->dsp == NULL) {
 	d->dsp = XOpenDisplay (d->name);
 	
@@ -495,6 +502,21 @@ gdm_slave_run (GdmDisplay *display)
 	}
     }
 
+    /* Set the busy cursor */
+    if (d->dsp != NULL) {
+	    Cursor xcursor = XCreateFontCursor (d->dsp, GDK_WATCH);
+	    XDefineCursor (d->dsp,
+			   DefaultRootWindow (d->dsp),
+			   xcursor);
+	    XFreeCursor (d->dsp, xcursor);
+	    XSync (d->dsp, False);
+    }
+
+    /* Just a race avoiding sleep, probably not necessary though,
+     * but doesn't hurt anything */
+    if ( ! d->handled)
+	    sleep (1);
+
     if (SERVER_IS_LOCAL (d)) {
 	    gdm_slave_send (GDM_SOP_START_NEXT_LOCAL, FALSE);
     }
@@ -503,7 +525,7 @@ gdm_slave_run (GdmDisplay *display)
 
     /* something may have gone wrong, try xfailed, if local (non-flexi),
      * the toplevel loop of death will handle us */ 
-    if (d->dsp == NULL) {
+    if (d->handled && d->dsp == NULL) {
 	    gdm_server_stop (d);
 	    if (d->type == TYPE_LOCAL)
 		    _exit (DISPLAY_XFAILED);
@@ -512,8 +534,8 @@ gdm_slave_run (GdmDisplay *display)
     }
 
     /* Some sort of a bug foo to make some servers work or whatnot,
-     * stolem from xdm sourcecode, perhaps not necessary, but can't hurt */
-    {
+     * stolen from xdm sourcecode, perhaps not necessary, but can't hurt */
+    if (d->handled) {
 	    Display *dsp = XOpenDisplay (d->name);
 	    if (dsp != NULL)
 		    XCloseDisplay (dsp);
@@ -530,14 +552,23 @@ gdm_slave_run (GdmDisplay *display)
     }
 
     /* checkout xinerama */
-    gdm_screen_init (d);
+    if (d->handled)
+	    gdm_screen_init (d);
 
     /* check log stuff for the server, this is done here
      * because it's really a race */
     if (SERVER_IS_LOCAL (d))
 	    gdm_server_checklog (d);
 
-    if (d->use_chooser) {
+    if ( ! d->handled) {
+	    /* yay, we now wait for the server to die,
+	     * which will in fact just exit, so
+	     * this code is a little bit too anal */
+	    while (d->servpid != 0) {
+		    select (0, NULL, NULL, NULL, NULL);
+	    }
+	    return;
+    } else if (d->use_chooser) {
 	    /* this usually doesn't return */
 	    gdm_slave_chooser ();  /* Run the chooser */
 	    return;
@@ -566,6 +597,10 @@ gdm_slave_run (GdmDisplay *display)
 		    gdm_verify_cleanup (d);
 		    _exit (DISPLAY_REMANAGE);
 	    }
+
+	    /* return to gdm_slave_start so that the server
+	     * can be reinitted and all that kind of fun stuff. */
+	    return;
     }
 
     if (gdm_first_login)
@@ -615,6 +650,9 @@ gdm_slave_run (GdmDisplay *display)
 		    gdm_slave_greeter_ctl_no_ret (GDM_ENABLE, "");
 		    gdm_slave_greeter_ctl_no_ret (GDM_RESETOK, "");
 	    }
+	    /* Note that greet is only true if the above was no 'login',
+	     * so no need to reinit the server nor rebake cookies
+	     * nor such nonsense */
     } while (greet);
 }
 
@@ -626,7 +664,6 @@ focus_first_x_window (const char *class_res_name)
 	pid_t pid;
 	Display *disp;
 	XWindowAttributes attribs = { 0, };
-	int i;
 
 	pid = fork ();
 	if (pid < 0) {
@@ -640,8 +677,7 @@ focus_first_x_window (const char *class_res_name)
 
 	closelog ();
 
-	for (i = 0; i < sysconf (_SC_OPEN_MAX); i++)
-	    close(i);
+	gdm_close_all_descriptors (0 /* from */, -1 /* except */);
 
 	/* No error checking here - if it's messed the best response
          * is to ignore & try to continue */
@@ -721,7 +757,6 @@ run_config (GdmDisplay *display, struct passwd *pwent)
 		return;
 	}
 	if (pid == 0) {
-		int i;
 		char **argv;
 		/* child */
 
@@ -729,7 +764,9 @@ run_config (GdmDisplay *display, struct passwd *pwent)
 		setgid (0);
 
 		/* setup environment */
-		gdm_clearenv_no_lang ();
+		/* FIXME: clearing environment is likely fairly stupid,
+		 * is there any reason we should do it anyway? */
+		/* gdm_clearenv_no_lang (); */
 
 		/* root here */
 		gnome_setenv ("XAUTHORITY", display->authfile, TRUE);
@@ -744,8 +781,7 @@ run_config (GdmDisplay *display, struct passwd *pwent)
 
 		closelog ();
 
-		for (i = 0; i < sysconf (_SC_OPEN_MAX); i++)
-			close(i);
+		gdm_close_all_descriptors (0 /* from */, -1 /* except */);
 
 		/* No error checking here - if it's messed the best response
 		 * is to ignore & try to continue */
@@ -1204,13 +1240,12 @@ run_pictures (void)
 		}
 		g_free (ret);
 
-		printf ("%c", STX);
-		fflush (stdout);
+		gdm_fdprintf (STDOUT_FILENO, "%c", STX);
 
 		i = 0;
 		while ((bytes = fread (buf, sizeof (char),
 				       sizeof (buf), fp)) > 0) {
-			fwrite (buf, 1, bytes, stdout);
+			write (STDOUT_FILENO, buf, bytes);
 			i += bytes;
 		}
 
@@ -1219,7 +1254,7 @@ run_pictures (void)
 		/* eek, this "could" happen, so just send some garbage */
 		while (i < s.st_size) {
 			bytes = MIN (sizeof (buf), s.st_size - i);
-			fwrite (buf, 1, bytes, stdout);
+			write (STDOUT_FILENO, buf, bytes);
 			i += bytes;
 		}
 			
@@ -1237,7 +1272,6 @@ gdm_slave_greeter (void)
     gint pipe1[2], pipe2[2];  
     gchar **argv;
     struct passwd *pwent;
-    int i;
     pid_t pid;
     
     gdm_debug ("gdm_slave_greeter: Running greeter on %s", d->name);
@@ -1279,8 +1313,7 @@ gdm_slave_greeter (void)
 
 	closelog ();
 
-	for (i = 2; i < sysconf (_SC_OPEN_MAX); i++)
-	    close(i);
+	gdm_close_all_descriptors (2 /* from */, -1 /* except */);
 
 	open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
 
@@ -1301,7 +1334,10 @@ gdm_slave_greeter (void)
 			    _("%s: Couldn't set userid to %d"),
 			    "gdm_slave_greeter", GdmUserId);
 	
-	gdm_clearenv_no_lang ();
+	/* FIXME: clearing environment is likely fairly stupid,
+	 * is there any reason we should do it anyway? */
+	/* gdm_clearenv_no_lang (); */
+
 	gnome_setenv ("XAUTHORITY", GDM_AUTHFILE (d), TRUE);
 	gnome_setenv ("DISPLAY", d->name, TRUE);
 
@@ -1428,14 +1464,15 @@ gdm_slave_greeter (void)
 	/* flush our input before we change the piping */
 	fflush (stdin);
 
-	if (pipe1[1] != STDOUT_FILENO) 
+	if (pipe1[1] != STDOUT_FILENO) {
 	    dup2 (pipe1[1], STDOUT_FILENO);
+	    close (pipe1[1]);
+	}
 	
-	if (pipe2[0] != STDIN_FILENO) 
+	if (pipe2[0] != STDIN_FILENO)  {
 	    dup2 (pipe2[0], STDIN_FILENO);
-
-	close (pipe1[1]);
-	close (pipe2[0]);
+	    close (pipe2[0]);
+	}
 	
 	gdm_debug ("gdm_slave_greeter: Greeter on pid %d", (int)pid);
 
@@ -1453,7 +1490,7 @@ parent_exists (void)
 {
 	pid_t ppid = getppid ();
 
-	if (ppid <= 0 ||
+	if (ppid <= 1 ||
 	    kill (ppid, 0) < 0)
 		return FALSE;
 	return TRUE;
@@ -1589,7 +1626,6 @@ gdm_slave_chooser (void)
 	struct passwd *pwent;
 	char buf[1024];
 	size_t bytes;
-	int i;
 	pid_t pid;
 
 	gdm_debug ("gdm_slave_chooser: Running chooser on %s", d->name);
@@ -1626,8 +1662,7 @@ gdm_slave_chooser (void)
 		closelog ();
 
 		close (0);
-		for (i = 2; i < sysconf (_SC_OPEN_MAX); i++)
-			close(i);
+		gdm_close_all_descriptors (2 /* from */, -1 /* except */);
 
 		open ("/dev/null", O_RDONLY); /* open stdin - fd 0 */
 		open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
@@ -1649,7 +1684,10 @@ gdm_slave_chooser (void)
 					_("%s: Couldn't set userid to %d"),
 					"gdm_slave_chooser", GdmUserId);
 
-		gdm_clearenv_no_lang ();
+		/* FIXME: clearing environment is likely fairly stupid,
+		 * is there any reason we should do it anyway? */
+		/* gdm_clearenv_no_lang (); */
+
 		gnome_setenv ("XAUTHORITY", GDM_AUTHFILE (d), TRUE);
 		gnome_setenv ("DISPLAY", d->name, TRUE);
 
@@ -2014,7 +2052,6 @@ session_child_run (struct passwd *pwent,
 {
 	sigset_t mask; 
 	int logfd;
-	int i;
 	gboolean failsafe = FALSE;
 	char *sesspath, *sessexec;
 	gboolean need_config_sync = FALSE;
@@ -2094,8 +2131,11 @@ session_child_run (struct passwd *pwent,
 		gnome_setenv ("LANG", language, TRUE);
 		gnome_setenv ("GDM_LANG", language, TRUE);
 	} else {
-		/* setusercontext sets up user languages */
-		gnome_setenv ("GDM_LANG", g_getenv ("LANG"), TRUE);
+		if (g_getenv ("LANG") == NULL)
+			gnome_unsetenv ("GDM_LANG");
+		else
+			/* setusercontext sets up user languages */
+			gnome_setenv ("GDM_LANG", g_getenv ("LANG"), TRUE);
 	}
 #else
 	if (setuid (pwent->pw_uid) < 0) 
@@ -2124,7 +2164,11 @@ session_child_run (struct passwd *pwent,
 	if (usrcfgok && savelang && home_dir_ok) {
 		gchar *cfgstr = g_strconcat ("=", home_dir,
 					     "/.gnome/gdm=/session/lang", NULL);
-		gnome_config_set_string (cfgstr, language);
+		/* we chose the system default language so wipe the lang key */
+		if (def_language)
+			gnome_config_clean_key (cfgstr);
+		else
+			gnome_config_set_string (cfgstr, language);
 		need_config_sync = TRUE;
 		g_free (cfgstr);
 	}
@@ -2146,8 +2190,7 @@ session_child_run (struct passwd *pwent,
 
 	closelog ();
 
-	for (i = 0; i < sysconf (_SC_OPEN_MAX); i++)
-	    close (i);
+	gdm_close_all_descriptors (0 /* from */, -1 /* except */);
 
 	/* No error checking here - if it's messed the best response
          * is to ignore & try to continue */
@@ -2447,8 +2490,7 @@ gdm_slave_session_start (void)
 
 	    g_free (language);
 
-	    if (lang != NULL &&
-		lang[0] != '\0') 
+	    if ( ! ve_string_empty (lang))
 		    language = g_strdup (lang);
 	    else
 		    language = g_strdup (GdmDefaultLocale);
@@ -2876,6 +2918,9 @@ gdm_slave_child_handler (int sig)
 		d->servstat = SERVER_DEAD;
 		d->servpid = 0;
 		gdm_server_wipe_cookies (d);
+
+		if ( ! d->handled)
+			_exit (DISPLAY_REMANAGE);
 	} else if (pid == extra_process) {
 		/* an extra process died, yay! */
 		extra_process = -1;
@@ -3040,7 +3085,7 @@ check_for_interruption (const char *msg)
 char * 
 gdm_slave_greeter_ctl (char cmd, const char *str)
 {
-    gchar buf[FIELD_SIZE];
+    char *buf = NULL;
     int c;
 
     /* There is no spoon^H^H^H^H^Hgreeter */
@@ -3051,18 +3096,18 @@ gdm_slave_greeter_ctl (char cmd, const char *str)
     check_notifies_immediately ++;
 
     if ( ! ve_string_empty (str)) {
-	    printf ("%c%c%s\n", STX, cmd, str);
+	    gdm_fdprintf (STDOUT_FILENO, "%c%c%s\n", STX, cmd, str);
     } else {
-	    printf ("%c%c\n", STX, cmd);
+	    gdm_fdprintf (STDOUT_FILENO, "%c%c\n", STX, cmd);
     }
-    fflush (stdout);
 
     /* Skip random junk that might have accumulated */
     do {
-	    c = getc (stdin);
+	    c = gdm_fdgetc (STDIN_FILENO);
     } while (c != EOF && c != STX);
     
-    if (c == EOF || fgets (buf, FIELD_SIZE-1, stdin) == NULL) {
+    if (c == EOF ||
+	(buf = gdm_fdgets (STDIN_FILENO)) == NULL) {
 	    check_notifies_immediately --;
 	    interrupted = TRUE;
 	    /* things don't seem well with the greeter, it probably died */
@@ -3073,15 +3118,10 @@ gdm_slave_greeter_ctl (char cmd, const char *str)
 
     check_for_interruption (buf);
 
-    /* don't forget to flush */
-    fflush (stdin);
-    
-    if (buf[0] != '\0') {
-	    int len = strlen (buf);
-	    if (buf[len-1] == '\n')
-		    buf[len-1] = '\0';
-	    return g_strdup (buf);
+    if ( ! ve_string_empty (buf)) {
+	    return buf;
     } else {
+	    g_free (buf);
 	    return NULL;
     }
 }
@@ -3168,7 +3208,6 @@ gdm_slave_exec_script (GdmDisplay *d, const gchar *dir, const char *login,
     const char *scr;
     gchar **argv;
     gint status;
-    int i;
 
     if (!d || !dir)
 	return EXIT_SUCCESS;
@@ -3193,8 +3232,7 @@ gdm_slave_exec_script (GdmDisplay *d, const gchar *dir, const char *login,
     case 0:
         closelog ();
 
-        for (i = 0; i < sysconf (_SC_OPEN_MAX); i++)
-	    close (i);
+	gdm_close_all_descriptors (0 /* from */, -1 /* except */);
 
         /* No error checking here - if it's messed the best response
          * is to ignore & try to continue */
@@ -3332,7 +3370,6 @@ gdm_parse_enriched_login (const gchar *s, GdmDisplay *display)
       if (pipe (pipe1) < 0) {
         gdm_error (_("gdm_parse_enriched_login: Failed creating pipe"));
       } else {
-        int i;
 	pid = gdm_fork_extra ();
 
         switch (pid) {
@@ -3347,10 +3384,7 @@ gdm_parse_enriched_login (const gchar *s, GdmDisplay *display)
 
 	    closelog ();
 
-	    for (i = 3; i < sysconf (_SC_OPEN_MAX); i++) {
-		    if (pipe1[1] != i)
-			    close (i);
-	    }
+	    gdm_close_all_descriptors (3 /* from */, pipe1[1] /* except */);
 
 	    openlog ("gdm", LOG_PID, LOG_DAEMON);
 
