@@ -381,7 +381,7 @@ gdm_verify_user (GdmDisplay *d,
     case PAM_PERM_DENIED :
 	gdm_error (_("User %s not permitted to gain access at this time"), login);
 	gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, 
-		_("\nThe system administrator has disabled access to the system temporary."));
+		_("\nThe system administrator has disabled access to the system temporarily."));
 	error_msg_given = TRUE;
 	goto pamerr;
     default :
@@ -557,14 +557,6 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display)
 	    goto setup_pamerr;
     }
 
-    /* Register the session */
-    pamerr = pam_open_session (pamh, 0);
-    if (pamerr != PAM_SUCCESS) {
-	    if (gdm_slave_should_complain ())
-		    gdm_error (_("Couldn't open session for %s"), login);
-	    goto setup_pamerr;
-    }
-
     /* Set credentials */
     pamerr = pam_setcred (pamh, PAM_ESTABLISH_CRED);
     if (pamerr != PAM_SUCCESS) {
@@ -573,6 +565,17 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display)
 	    goto setup_pamerr;
     }
 
+    /* Register the session */
+    pamerr = pam_open_session (pamh, 0);
+    if (pamerr != PAM_SUCCESS) {
+	    /* Throw away the credentials */
+	    pam_setcred (pamh, PAM_DELETE_CRED);
+
+	    if (gdm_slave_should_complain ())
+		    gdm_error (_("Couldn't open session for %s"), login);
+	    goto setup_pamerr;
+    }
+
     /* Workaround to avoid gdm messages being logged as PAM_pwdb */
     closelog ();
     openlog ("gdm", LOG_PID, LOG_DAEMON);
