@@ -98,6 +98,7 @@
 #include "xdmcp.h"
 
 gint pending = 0;
+int gdm_xdmcpfd = -1;
 
 #ifdef HAVE_LIBXDMCP
 
@@ -105,7 +106,6 @@ gint pending = 0;
 gint allow_severity = LOG_INFO;
 gint deny_severity = LOG_WARNING;
 
-static gint gdm_xdmcpfd = -1;
 static guint xdmcp_source = 0;
 static gint globsessid;
 static gchar *sysid;
@@ -241,8 +241,7 @@ gdm_xdmcp_init (void)
     /* Fetch and store local hostname in XDMCP friendly format */
     if (gethostname (hostbuf, 255) != 0) {
 	gdm_error (_("gdm_xdmcp_init: Could not get server hostname: %s!"), strerror (errno));
-	GdmXdmcp = FALSE;
-	return FALSE;
+	strcmp (hostbuf, "localhost.localdomain");
     }
 
     if ( ! initted) {
@@ -287,6 +286,7 @@ gdm_xdmcp_run (void)
 	GIOChannel *xdmcpchan;
 
 	xdmcpchan = g_io_channel_unix_new (gdm_xdmcpfd);
+	g_io_channel_set_encoding (xdmcpchan, NULL, NULL);
 	g_io_channel_set_buffered (xdmcpchan, FALSE);
 
 	xdmcp_source = g_io_add_watch_full
@@ -1532,6 +1532,8 @@ gdm_xdmcp_display_alloc (struct in_addr *addr, const char *hostname, gint displa
     d->acctime = time (NULL);
     d->dispnum = displaynum;
 
+    d->handled = TRUE;
+
 #ifdef __linux__
     d->vt = -1;
 #endif
